// -*-C++-*-
/* 
 * This is a drastically modified piece of example code from one of the 
 * tutorials. It is set up to test the numberScroller class. It puts 3 
 * numberScrollers in the bottom panel of a window, and uses the events
 * posted by 2 of them to make them stay synchronized. The three also 
 * represent 3 boundary conditions for the numberScroller class: one has 
 * a range that extends into the negative numbers, the second has a range
 * that includes only 1 number (500), and the third has a range that is 
 * entered into the constructor backwards.
 * 
 * This code was hacked to pieces by Tim Farnum, who wrote the numberScroller
 * widget. It isn't warranted to stay on a disk drive for 10 minutes, much less
 * do anything productive.
 */

import java.awt.*;
import numberScroller;

public class nsWindow extends Frame {
  boolean inAnApplet = true;
  private numberScroller nsNeg;
  private numberScroller nsBak;
    
  final String FILEDIALOGMENUITEM = "File dialog...";

  public nsWindow() {
    Panel bottomPanel = new Panel();
    Panel centerPanel = new Panel();

    MenuBar mb = new MenuBar();
    Menu m = new Menu("Menu");
    m.add(new MenuItem("Menu item 1"));
    m.add(new CheckboxMenuItem("Menu item 2"));
    m.add(new MenuItem("Menu item 3"));
    m.add(new MenuItem("-"));
    m.add(new MenuItem(FILEDIALOGMENUITEM));
    m.add(new MenuItem("-"));
    m.add(new MenuItem("Quit"));
    mb.add(m);
    setMenuBar(mb);

    //Add small things at the bottom.
    bottomPanel.setLayout(new FlowLayout());
    nsNeg = new numberScroller("negScroll", -150, 150, 0, 10);
    nsBak = new numberScroller("bakScroll", 500, 200, 300, 25);
    bottomPanel.add(nsNeg);
    bottomPanel.add(new numberScroller("nulScroll", 500, 500, 500, 5));
    bottomPanel.add(nsBak);

    //Add big things to the center area.
    centerPanel.setLayout(new BorderLayout());
    //Put a canvas in the left column.
    centerPanel.add("Center",new MyCanvas());
    setLayout(new BorderLayout());
    add("South", bottomPanel);
    add("Center", centerPanel);
  }

  public boolean handleEvent(Event evt) {
    switch(evt.id) {
    case Event.ACTION_EVENT: {
      if(evt.target instanceof MenuItem) {
	if((String)evt.arg == FILEDIALOGMENUITEM) {
	  FileDialog fd = new FileDialog(this, "FileDialog");
	  fd.show();
	  return true;
	}
	if((String)evt.arg == "Quit") {
	  if(inAnApplet) {
	    dispose();
	  } else {
	    System.exit(0);
	  }
	  return true;  //shouldn't ever reach this
	}
	return false;  //if we reach here, we haven't handled it yet.
      }//if(evt.target instanceof MenuItem)
      if(evt.target instanceof numberScroller) {
	if(evt.target == nsNeg) {
	  nsBak.setValue(nsNeg.getValue() + 350);
	  return true;
	} else if(evt.target == nsBak) {
	  nsNeg.setValue(nsBak.getValue() - 350);
	  return true;
	}
	return false;
      }//if(evt.target instanceof numberScroller)
      return false;
    }//case Event.ACTION_EVENT:
    
    case Event.WINDOW_DESTROY: {
      if(inAnApplet) {
	dispose();
      } else {
	System.exit(0);
      }
      return true;  //shouldn't ever reach this
    }//case Event.WINDOW_DESTROY:

    default: {
      return false;
    }
    }//switch(evt.id)
  }//handleEvent

  public static void main(String args[]) {
    nsWindow window = new nsWindow();
    window.inAnApplet = false;

    //window.setTitle(window.getClass().getName() + " Application");
    window.setTitle("numScroll etc.");
    window.pack();
    window.show();
  }
}


class MyCanvas extends Canvas {

  public void paint(Graphics g) {
    int w = size().width;
    int h = size().height;
    int i,wj,hj,x1,y1,x2,y2;
    g.drawRect(0, 0, w - 1, h - 1);
    wj = w / 25;
    hj = h / 25;
    x1 = w; y1 = 0; x2 = 0; y2 = 0;
    for(; x1 > 0;) {
      g.drawLine(x1,y1,x2,y2);
      x1 -= wj; y2 += hj;
    }
  }

    public Dimension minimumSize() {
    return new Dimension(150,130);
    }

    public Dimension preferredSize() {
    return minimumSize();
    }
}
