// -*-C++-*- (I'm just learning about emacs)
/** numberScroller class
 *
 * An awt widget to allow a small scroll bar to control a number in a text field  
 * and vice versa. 
 *
 * Constructor numberScroller(String labl, 
 *		     	      int minNum,
 *			      int maxNum, 
 *			      int startNum, 
 *			      int jumpNum)
 *   -labl = label for the scroller
 *   -minNum = minimum value the scroller can take
 *   -maxNum = maximum value the scroller can take
 *   -startNum = initial value of the scroller
 *   -jumpNum = how much the scroller changes for page-clicks on the scrollbar
 *
 * Methods
 *   -int getValue() returns the current value
 *   -Label getLabel() returns the label 
 *   -int setValue(int i) enforces minNum <= i <= maxNum, sets current value to i,
 *    and returns the current value.
 *
 * Events
 *   -posts an event to its parent when the scrollbar or textField change.
 *    The event is an ACTION_EVENT, with Event.arg set to the new current value
 *    in Integer format. (Don't get me started about the Number class heirarchy.) 
 *
 * This class was written by Tim Farnum (tfarnum@rpa.net). It is in the public domain.
 * If you have any feedback on the style or execution of this design, drop me a line.
 * If you feel like giving me credit, go ahead. There is no warranty of anything, not
 * even death and taxes. Use this at your own risk. Change it if you want to. If you
 * don't like it, erase it.
 *
 */ 

import java.awt.*;


public class numberScroller extends Panel{
  private Label myLabel;
  private TextField myText;
  private Scrollbar myScroll;
  private int current, minN, maxN;

  public int getValue() { return current; }
  public Label getLabel() { return myLabel; }

  public int setValue(int i) { 
    if(i < minN) i = minN;      //set i to at least minN or at most maxN
    else
      if(i > maxN) i = maxN;
    current = i;
    myText.setText(String.valueOf(current));
    myScroll.setValue(maxN-current);
    return current;
  }

  public numberScroller(String labl, 
		     	int minNum,
			int maxNum, 
			int startNum, 
			int jumpNum) {
    super();
    current = startNum;
    if(maxNum > minNum) {              //make sure maxN is bigger than minN
      minN = minNum;
      maxN = maxNum;
    } else if(maxNum <= minNum) {
      minN = maxNum;
      maxN = minNum;
    }

    if(startNum < minN) { startNum = minN; }  //put startNum in range
    else if(startNum > maxN) { startNum = maxN; }
    
    myLabel = new Label(labl);

    myText = new TextField(new Integer(startNum).toString(), 
			   new Integer(maxNum).toString().length());
    myScroll = new Scrollbar(Scrollbar.VERTICAL, 
			     maxN-startNum,
			     jumpNum,
			     0,           //these are different bec. scroll is upside-
			     maxN-minN);  //down for this application
    
    GridBagLayout gbl = new GridBagLayout();  //using GridBag to try squishing Scrollbar.
    setLayout(gbl);                           //It doesn't work. Is there a way to make 
    GridBagConstraints gbc = new GridBagConstraints();  //the scrollbar shorter?
    gbc.weightx = 1.0;
    gbc.insets = new Insets(2,2,2,0);     //space for an outline
    gbl.setConstraints(myLabel, gbc);     //but not extra space within the widget   
    gbc.insets = new Insets(2,0,2,0);
    gbl.setConstraints(myText, gbc);
    gbc.insets = new Insets(2,0,2,2);
    gbl.setConstraints(myScroll, gbc);

    add(myLabel);
    add(myText);
    add(myScroll);
  }//public numberScroller()
    
  public void paint(Graphics g) {
    Dimension d = size();
    g.drawRect(0,0, d.width - 1, d.height - 1);
  }

    /** Respond to user actions. */
  public boolean handleEvent(Event ev) {
    int temp;
    switch(ev.id) {
    case Event.ACTION_EVENT: {
      if(ev.target instanceof TextField) {
	try {
	  temp = Integer.parseInt(myText.getText());
	} catch(NumberFormatException e) {
	  //System.out.println("NumberFormatException: "+ e.getMessage());
	  temp = current;
	}
	setValue(temp);  //use setValue because the textField may need changing, too
	postAnEvent();   //if the value typed in was not valid
	return true;
      }//if(ev.target instanceof TextField)
      return false;
    }//case action events

    case Event.SCROLL_ABSOLUTE: 
    case Event.SCROLL_LINE_UP: 
    case Event.SCROLL_LINE_DOWN: 
    case Event.SCROLL_PAGE_UP: 
    case Event.SCROLL_PAGE_DOWN: {
      current =  maxN - myScroll.getValue();
      myText.setText(String.valueOf(current));  //no need for setValue here, because 
      postAnEvent();                            //the scrollbar is already constrained.
      return true;
    }//case scroll events

    default: {
      return false;
    }
    }//switch(ev.id())
  }//handleEvent()

  private void postAnEvent() {
    getParent().postEvent(new Event(this, Event.ACTION_EVENT, new Integer(current)));
  }

}//public class numberScroller

    
      
		

